'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _bluebird = require('bluebird');

var _logging = require('./util/logging');

var _prompt = require('./util/prompt');

var _prompt2 = _interopRequireDefault(_prompt);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//
// This is the entry script for the CLI utility.
//

//region Imports

var c = require('chalk');
//endregion

process.on('unhandledRejection', function (e) {
  throw e;
});

function cli() {
  var _this = this;

  main().then((0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee() {
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, _this);
  })));
  return true;
}

module.exports = cli;

////////////////////////////////////////////////////////////////////////////////

var main = function () {
  var ref = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee2() {
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:

            welcome();
            _context2.next = 3;
            return confirmLocation(1);

          case 3:
            initPackageJson(2);
            _context2.next = 6;
            return addGulpScriptToPackageJson(3);

          case 6:
            _context2.next = 8;
            return createGulpfile(4);

          case 8:
            _context2.next = 10;
            return installModules(5);

          case 10:
            gulpBootstrap(6);
            printHelp(7);

          case 12:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));
  return function main() {
    return ref.apply(this, arguments);
  };
}();

////////////////////////////////////////////////////////////////////////////////
// Steps
////////////////////////////////////////////////////////////////////////////////

// TODO
// - Use init-package-json.

function welcome(order) {

  (0, _logging.log)(c.bold('Welcome to the Babelator CLI'));
  (0, _logging.log)(c.grey('- Babelator will first create a minimal npm module with gulp installed, and Babelator\'s gulp tasks available.'));
  (0, _logging.log)(c.grey('- The rest of Babelator\'s bootstrapping occurs through gulp tasks by running `gulp bootstrap`.'));
  (0, _logging.log)(c.grey('- What about existing files? Babelator will confirm all changes to existing files making it super safe and easy to babelate existing projects.'));
}

var confirmLocation = function () {
  var ref = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee3(order) {
    var inquirer, cwd, ans;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:

            (0, _logging.logHeading)(order + '. Confirm module location');

            inquirer = require('inquirer');
            cwd = require('cwd');
            _context3.next = 5;
            return (0, _prompt2.default)([{ type: 'confirm', name: 'confirm', message: 'Confirm your module\'s location: \'' + cwd() + '/package.json\'' }]);

          case 5:
            ans = _context3.sent;

            if (!ans.confirm) {
              (0, _logging.log)('Exiting');
              process.exit(1);
            }

          case 7:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));
  return function confirmLocation(_x) {
    return ref.apply(this, arguments);
  };
}();

function initPackageJson(order) {

  (0, _logging.logHeading)(order + '. Create package.json if it does not exist');

  var fs = require('fs');
  var cwd = require('cwd');

  var _require = require('child_process');

  var spawnSync = _require.spawnSync;

  var packageJsonExists = fs.existsSync(cwd('package.json'));
  if (!packageJsonExists) {
    console.log('File `package.json` does not exist. A basic file will be created now. Don\'t worry, the contents will be iteratively modified further on.');
    (0, _logging.log)('=> npm init -y');
    // NOTE: -y - Use defaults and do not prompt user.
    spawnSync('npm', ['init', '-y'], { stdio: 'inherit' });
  } else {
    (0, _logging.log)('File `package.json` already exists.');
  }
}

var addGulpScriptToPackageJson = function () {
  var ref = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee4(order) {
    var _require2, promisify, modifyPackageJson, _;

    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:

            (0, _logging.logHeading)(order + '. Add gulp command to package.json scripts hash');

            _require2 = require('bluebird');
            promisify = _require2.promisify;
            modifyPackageJson = require('./util/modifyPackageJson');
            _ = require('lodash');
            _context4.next = 7;
            return promisify(modifyPackageJson)(function (json) {
              _.set(json, 'scripts.gulp', './node_modules/gulp/bin/gulp.js');
              return json;
            });

          case 7:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));
  return function addGulpScriptToPackageJson(_x2) {
    return ref.apply(this, arguments);
  };
}();

var useWebpackerator = undefined;

var createGulpfile = function () {
  var ref = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee5(order) {
    var _require3, promisify, replaceFile, cwd, _require4, confirm, gulpfileContents;

    return _regenerator2.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:

            (0, _logging.logHeading)(order + '. Add Babelator tasks to gulpfile.js');

            _require3 = require('bluebird');
            promisify = _require3.promisify;
            replaceFile = require('./util/replaceFile');
            cwd = require('cwd');
            _require4 = require('inquirer-promise');
            confirm = _require4.confirm;
            _context5.next = 9;
            return confirm('Do you want to use webpack via webpackerator?');

          case 9:
            useWebpackerator = _context5.sent;
            gulpfileContents = undefined;

            gulpfileContents = "var gulp = require('gulp')\n";
            if (useWebpackerator) gulpfileContents += "require('webpackerator/defaults/gulp')(gulp)\n";
            // babelator comes last so it can use all gulp files for bootstrap.
            gulpfileContents += "require('babelator')(gulp)\n";

            _context5.next = 16;
            return promisify(replaceFile)(cwd('gulpfile.js'), gulpfileContents);

          case 16:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, this);
  }));
  return function createGulpfile(_x3) {
    return ref.apply(this, arguments);
  };
}();

var installModules = function () {
  var ref = (0, _bluebird.coroutine)(_regenerator2.default.mark(function _callee6(order) {
    var devDependencies, dependencies;
    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:

            // TODO(vjpr): Set to current versions from npm.
            devDependencies = [['babelator', '0.2.8'], ['gulp', 'gulpjs/gulp.git#4.0'], ['babel-preset-live', '0.0.4']];
            dependencies = [['babel-register', '^6'], // For `mocha:src` tests and if we add a `modules` dir.
            ['babel-runtime', '^5'], ['bluebird', '^3'], ['babel-polyfill', '^6']];

            //////////////////////////////////////////////////////////////////////////////

            if (useWebpackerator) {
              devDependencies = devDependencies.concat([['webpackerator', '^0.1.0'], ['babel-preset-react-hmre', '1.1.1'], ['webpack-dev-server', '1.14.1']]);
              dependencies = dependencies.concat([['babel-loader', '^6']]);
            }

            //////////////////////////////////////////////////////////////////////////////

            (0, _logging.logHeading)(order + '. Add npm dependencies to package.json');

            _context6.next = 6;
            return require('npm-install-prompt').addNpmDependenciesToPackageJson({
              devDependencies: devDependencies,
              dependencies: dependencies,
              reject: {
                devDependencies: ['babel-preset-react', 'babel-preset-es2015', 'babel-preset-stage-0'],
                dependencies: ['babel-preset-react', 'babel-preset-es2015', 'babel-preset-stage-0']
              }
            });

          case 6:

            //////////////////////////////////////////////////////////////////////////////

            (0, _logging.logHeading)(order + '. Install npm dependencies');

            (0, _logging.log)(c.grey('- `npm install` takes some time. To speed up the process you can use `npm link` instead.'));
            (0, _logging.log)(c.grey('- For this to work you need to checkout the repo and `npm link` it.'));
            (0, _logging.log)(c.grey('  - E.g. `git checkout https://github.com/vjpr/babelator && npm install && npm link`'));
            (0, _logging.log)(c.grey('- For monorepos like babel you need to `npm link` the individual packages.'));
            (0, _logging.log)(c.grey('  - E.g. `git checkout https://github.com/babel/babel && cd packages/babel-preset-es2015 && npm install && npm link`'));
            (0, _logging.log)(c.grey('Tip: Use `npm_lazy` module to locally cache npm modules to speed up installs.'));
            (0, _logging.log)();

            _context6.next = 16;
            return require('npm-install-prompt').installNpmDependencies({ devDependencies: devDependencies, dependencies: dependencies });

          case 16:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, this);
  }));
  return function installModules(_x4) {
    return ref.apply(this, arguments);
  };
}();

function gulpBootstrap(order) {
  var _require5 = require('child_process');

  var spawnSync = _require5.spawnSync;

  var cmd = 'npm run gulp bootstrap';

  (0, _logging.log)();
  (0, _logging.log)('=>', cmd);

  spawnSync('npm', ['run', 'gulp', 'bootstrap'], { stdio: 'inherit' });
}

function printHelp(order) {

  // TODO
  (0, _logging.padlog)('Bootstrapping complete!');

  (0, _logging.log)(c.bold.yellow('Getting started'));
  (0, _logging.log)();
  (0, _logging.log)('  npm run gulp test-watch ' + c.grey('Compiles and run tests on file change. Useful for development!'));
  (0, _logging.log)('  npm run gulp nodemon ' + c.grey('Run package.json main script with nodemon'));
  (0, _logging.log)('  npm run gulp release ' + c.grey('Interactive CLI to release new version on npm'));
  (0, _logging.log)('  npm-check -us ' + c.grey('Interactively update all your dependencies to their latest version'));
  (0, _logging.log)();
}