'use strict';

// TODO(vjpr): Use https://github.com/slushjs/gulp-conflict/blob/master/index.js

//region Imports
var fs = require('fs');
var chalk = require('chalk');
var _console = console;
var log = _console.log;

var path = require('path');
var cwd = require('cwd');
//endregion

module.exports = function (dest, newContents, done) {

  var diff = require('diff');

  var _require = require('inquirer-promise');

  var confirm = _require.confirm;

  if (!fs.existsSync(dest)) {
    fs.writeFileSync(dest, newContents, 'utf8');
    log('Created new file: %s', dest);
    log();
    log(chalk.green(newContents));
    log();
    return done();
  }

  var contents = fs.readFileSync(dest, 'utf8');

  var delta = diff.diffChars(contents, newContents);

  //log({contents, newContents, delta})
  if (delta.length <= 1) {
    log('Skipping ' + chalk.magenta(path.relative(cwd(), dest)) + ' (identical)');
    return done();
  }

  log('Proposed changes to:', dest);

  log();
  printDiff(delta);
  log();

  confirm('Write?').then(function (yes) {
    if (!yes) return done();
    fs.writeFileSync(dest, newContents, 'utf8');
    log(chalk.green('Wrote file.'));
    done();
  });
};

function printDiff(delta) {
  //console.log(delta)
  delta.forEach(function (part) {
    // green for additions, red for deletions
    // grey for common parts
    var color = part.added ? 'green' : part.removed ? 'red' : 'grey';
    process.stderr.write(chalk[color](part.value));
  });
}